package com.biganto.visual.roompark.conductor.dialogs.tour_chooser

import android.content.Context
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import androidx.annotation.LayoutRes
import androidx.recyclerview.widget.DividerItemDecoration
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import butterknife.BindView
import com.biganto.visual.roompark.R
import com.biganto.visual.roompark.conductor.dialogs.change_handler.DialogChangeHandler
import com.biganto.visual.roompark.domain.model.TourModel
import com.biganto.visual.roompark.presentation.screen.settings.util.CommonRecyclerAdapter
import com.biganto.visual.roompark.presentation.screen.settings.util.CommonViewHolder
import com.biganto.visual.roompark.util.extensions.setGone
import com.biganto.visual.roomparkvr.data.repository.db.requery.model.DownloadState
import com.bluelinelabs.conductor.Controller
import com.bluelinelabs.conductor.RouterTransaction
import com.bumptech.glide.Glide
import com.bumptech.glide.load.engine.DiskCacheStrategy
import com.google.android.material.textview.MaterialTextView
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.disposables.CompositeDisposable
import java.util.concurrent.TimeUnit
import javax.inject.Inject


/**
 * Created by Vladislav Bogdashkin on 09.04.2019.
 */

internal const val  TOUR_MODEL_LIST = "CHOOSE_TOUR_LIST_KEY"

private fun formBundle(photos: ArrayList<TourModel>): Bundle {
    val b = Bundle()
    b.putParcelableArrayList(TOUR_MODEL_LIST,photos)
    return b
}

class ChooseTourDialogController : Controller {

    constructor(args: Bundle) : super(args)

    constructor(items: ArrayList<TourModel>) : super(formBundle(items))

    private lateinit var recyclerView : RecyclerView

    private val detachDisposable = CompositeDisposable()

    override fun onDetach(view: View) {
        detachDisposable.clear()
        super.onDetach(view)
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup): View {
        val view = inflater.inflate(getLayoutId(), container, false)

        recyclerView = view.findViewById(R.id.toursList)
        recyclerView.layoutManager =
            LinearLayoutManager(activity, RecyclerView.VERTICAL, false)
        recyclerView.adapter = TourChooserAdapter()
        recyclerView.itemAnimator = null
        recyclerView.addItemDecoration(
            DividerItemDecoration(activity,DividerItemDecoration.VERTICAL)
        )

        args.getParcelableArrayList<TourModel>(TOUR_MODEL_LIST)?.let {
            (recyclerView.adapter as TourChooserAdapter).setItems(it)
        }

        detachDisposable.add(
        (recyclerView.adapter as TourChooserAdapter)
            .onItemClicked
            .debounce(300L,TimeUnit.MILLISECONDS)
            .observeOn(AndroidSchedulers.mainThread())
            .subscribe(::onTourClicked)
        )

        //view.findViewById<View>(R.id.close_current_button).setOnClickListener { handleBack() }
        view.setOnClickListener { handleBack() }

        return view
    }


    private fun onTourClicked(tour: TourModel) {
        when (tour.downloadState) {
            DownloadState.Downloaded -> {
                activity?.let { startPlayer(it, tour) }
                handleBack()
            }
            else -> {
                router.replaceTopController(
                    RouterTransaction.with(DownloadTourDialogController(tour))
                        .popChangeHandler(DialogChangeHandler())
                        .pushChangeHandler(DialogChangeHandler())
                )
            }
        }
    }

    @LayoutRes
    fun getLayoutId() = R.layout.tours_chooser_screen

    override fun handleBack(): Boolean {
        return router.popCurrentController()
    }
}

internal class TourChooserAdapter:CommonRecyclerAdapter<TourChooserViewHolder,TourModel>(){
    override val vhKlazz = TourChooserViewHolder::class
    override fun getVhLayout(): Int = R.layout.tour_chooser_viewholder
}

internal class TourChooserViewHolder(itemView: View) :CommonViewHolder<TourModel>(itemView){

    @BindView(R.id.tour_preview_imageView)
    lateinit var tourPreview : ImageView

    @BindView(R.id.tour_name)
    lateinit var tourName : MaterialTextView

    @BindView(R.id.tour_status_imageView)
    lateinit var tourStatus : ImageView

    @Inject
    lateinit var activity:Context

    override fun onViewBound(model: TourModel) {
        tourName.text = model.title
        Glide.with(tourPreview)
            .load(model.previewUrl)
            .diskCacheStrategy(DiskCacheStrategy.ALL)
            .into(tourPreview)

        tourStatus.setGone(model.downloadState == DownloadState.Downloaded)

        tourStatus.setImageDrawable(
            itemView.context.resources.getDrawable(
                when(model.downloadState){
                    DownloadState.Downloaded -> R.drawable.ic_download
                    DownloadState.Suspended -> R.drawable.ic_download
                    DownloadState.DownloadQueue -> R.drawable.ic_download
                    else -> R.drawable.ic_download
                }
                ,null)
        )
    }

}
