package com.biganto.visual.roompark.data.repository.api.biganto

import com.biganto.visual.roompark.data.repository.api.biganto.raw.TourFilesDataRaw
import com.biganto.visual.roompark.data.repository.api.biganto.raw.TourPreviewRaw
import io.reactivex.*
import io.reactivex.schedulers.Schedulers
import okhttp3.HttpUrl
import okhttp3.ResponseBody
import retrofit2.Response
import retrofit2.Retrofit
import timber.log.Timber
import timber.log.Timber.e
import javax.inject.Inject
import javax.inject.Named
import javax.inject.Singleton


/**
 * Created by Vladislav Bogdashkin on 13.06.2018.
 */

@Singleton
class BigantoRetrofitRepository  @Inject constructor(@Named("bigantoApi") retrofit:Retrofit)
    : IBigantoApi{

    private val api = retrofit.create(IBigantoMobileApi::class.java)


    override fun provideHttpUrl(): HttpUrl {
        return HttpUrl.parse(IBigantoMobileApi.BASE_URL)!!
    }

    override fun getAppVersion() = api
            .getAppVersion()
            .compose(RetrofitResponseValidation())

    override fun getToursPreviewById(tourIds: List<String>): Observable<List<TourPreviewRaw>> {
        return api.getToursPreviewById(ids = tourIds.joinToString(","))
                .compose(RetrofitResponseValidation())
                .subscribeOn(Schedulers.io())
    }

    override fun getTourMetaAsString(tour_id: String): Observable<String> = api
            .getTourMetaAsString(ids = tour_id)
            .map { it.toString() }
            .doOnError { e(it) }
            .subscribeOn(Schedulers.io())


    override fun getTourFiles(tour_id: String, resolution: String): Observable<List<TourFilesDataRaw>> = api
            .getTourFiles(ids = tour_id, resolution = resolution)
            .compose(RetrofitResponseValidation())
            .doOnError { e(it) }
            .subscribeOn(Schedulers.io())


    override fun downloadFile(uri: String, headers: Map<String, String>?): Flowable<ResponseBody> = api
            .requestFile(headers ?: HashMap<String, String>(), uri)
            .compose(FlowableRetrofitResponseValidation())
            .doOnError(::e)


    override fun getOfferTours(multiTourId:Int): Observable<List<TourPreviewRaw>> =
        api
        .getOfferTours(offerId = multiTourId)

        .compose(RetrofitResponseValidation())

        .map { it[multiTourId.toString()]?.toList()?: error("No tours avaliable")}
        .doOnError { e(it) }
        .subscribeOn(Schedulers.io())
}


internal class RetrofitResponseValidation<T> : ObservableTransformer<Response<T>, T> {
    override fun apply(responseObservable: Observable<Response<T>>): ObservableSource<T> {
        return responseObservable.switchMap { resp ->
//            Timber.d("reutrned code: %s",resp.code())
            if (resp.code() == 200 || resp.code() == 206)
                if (resp.body() == null)
                    return@switchMap Completable.complete().toObservable<T>().doOnNext{Timber.d("Completed responseBody")}
                else return@switchMap Observable.just(resp.body())
            Timber.d("Returning Exception!")
            return@switchMap Observable.error<T>(retrofit2.HttpException(resp))
        }
    }
}


internal class RetrofitResponseValidationString<String> : ObservableTransformer<Response<String>, String> {
    override fun apply(responseObservable: Observable<Response<String>>): ObservableSource<String> {
        return responseObservable.switchMap<String> { resp ->
//            Timber.d("reutrned code: %s",resp.code())
            if (resp.code() == 200)
                if (resp.body() == null)
                    return@switchMap Completable.complete().toObservable<String>().doOnNext{Timber.d("Completed responseBody")}
                else return@switchMap Observable.just<String>(resp.body())
            Timber.d("Returning Exception!")
            return@switchMap Observable.error<String>(retrofit2.HttpException(resp))
        }
    }
}



internal class FlowableRetrofitResponseValidation<T> : FlowableTransformer<Response<T>, T> {
    override fun apply(responseObservable: Flowable<Response<T>>): Flowable<T> {
        return responseObservable.switchMap { resp ->
            if (resp.code() == 200 || resp.code() == 206)
                if (resp.body() == null)
                    return@switchMap Completable.complete().toFlowable<T>().doOnNext{Timber.d("Completed responseBody")}
                else return@switchMap Flowable.just(resp.body())
            Timber.d("Returning Exception!")
            return@switchMap Flowable.error<T>(retrofit2.HttpException(resp))
        }
    }
}


//Allow to override HttpException with custom exception output, example:

//sealed class CustomApiException(val code:Int,val biganto_message:String):RuntimeException()
//{
//    class RedirectException():CustomApiException(code = 300,biganto_message = "Redirect page..")
//    class WrongAuthDataException():CustomApiException(code = 401,biganto_message = "Login or password not allowed")
//    class BlockedUserException():CustomApiException(code = 403,biganto_message = "User has been banned")
//    class ServerException():CustomApiException(code = 500,biganto_message = "Error server: undefined")
//
//}