package com.biganto.visual.roompark.data.repository.db.requrey

import android.app.Application
import com.biganto.visual.roompark.Models
import com.biganto.visual.roompark.data.repository.db.IDb
import com.biganto.visual.roompark.data.repository.db.requrey.model.*
import com.biganto.visual.roompark.di.dagger.DATABASE_VERSION
import com.biganto.visual.roomparkvr.data.repository.db.requery.model.DownloadState
import com.biganto.visual.roomparkvr.data.repository.db.requery.model.TourPreview
import com.biganto.visual.roomparkvr.data.repository.db.requery.model.TourPreviewEntity
import dagger.Module
import dagger.Provides
import io.reactivex.BackpressureStrategy
import io.reactivex.Completable
import io.reactivex.Observable
import io.reactivex.Single
import io.reactivex.rxkotlin.toCompletable
import io.reactivex.schedulers.Schedulers
import io.requery.Persistable
import io.requery.android.sqlite.DatabaseSource
import io.requery.reactivex.KotlinReactiveEntityStore
import io.requery.reactivex.ReactiveResult
import io.requery.sql.KotlinEntityDataStore
import io.requery.sql.TableCreationMode
import timber.log.Timber
import javax.inject.Inject

/**
 * Created by Vladislav Bogdashkin on 13.06.2018.
 */


@Module
class DbModule{

        @Provides
        fun provideStore(context: Application): KotlinReactiveEntityStore<Persistable> {
                Timber.d("Kotlin store creating..")
                val source = DatabaseSource(context, Models.DEFAULT, "BigantoPerfect", DATABASE_VERSION)

//                source.setLoggingEnabled(true)
//                source.setWriteAheadLoggingEnabled(true)
                source.setTableCreationMode(TableCreationMode.DROP_CREATE)
                val store = KotlinEntityDataStore<Persistable>(source.configuration)
                Timber.d("Kotlin store %s",source)
                return KotlinReactiveEntityStore(store)
        }
}

class RequeryRepository @Inject constructor(
        private val store: KotlinReactiveEntityStore<Persistable>
)
: IDb {

        override fun dropTourTable() = store.delete(TourPreviewEntity::class).get().toCompletable()

        override fun dropTourFileJuncTable() = store.delete(TourFileJunctionEntity::class).get().toCompletable()

        override fun dropFileTable() = store.delete(FileEntity::class).get().toCompletable()

        private inline fun <reified T : Persistable> fetchAll() =
                store.select(T::class)

        override fun upsertUser(entity: UserEntity): Observable<UserEntity> =
                store.upsert(entity).toObservable()

        override fun <T : Persistable> upsert(entity: T): Single<T>  = store.upsert(entity)

        override fun <T : List<Persistable>> upsert(entity: T): Single<Iterable<Persistable>> =
                store.upsert(entity)

        override fun <T : List<Persistable>> blockingUpsert(entity: T) =
                store.upsert(entity).toObservable().blockingSubscribe()

        override fun upsertEstate(entity:EstateEntity) =
                store.upsert(entity).toObservable().blockingSubscribe()

        override fun fetchFeeds(): Observable<FeedEntity> =
                fetchAll<FeedEntity>().get().observable()

        override fun fetchEstateByNumber(building:Int,number:String): ReactiveResult<EstateEntity> =
                store.select(EstateEntity::class)
                        .where(EstateEntity.NUMBER.eq(number))
                        .and(EstateEntity.INFO_BUILDING.eq(building))
                        .get()

        override fun getFeed(feedAlias: String): ReactiveResult<FeedEntity> =
                fetchAll<FeedEntity>().where(FeedEntity.ALIAS.eq(feedAlias))
                        .get()

        override fun getArticle(id: Int): ReactiveResult<ArticleEntity> =
                fetchAll<ArticleEntity>().where(ArticleEntity.ID.eq(id))
                        .get()

        override fun getTopLevelAlbums(): Observable<List<ImageAlbumEntity>> =
                Observable.fromArray(
                        store.raw(
                                ImageAlbumEntity::class,
                                "SELECT alb.* from ImageAlbum alb " +
                                        "left join ImageAlbumJunction albJun on albJun.albumId = alb.id " +
                                        "WHERE albJun.albumId is null "
                        ).toList()
                )

        override fun getChildAlbums(parentId: Int): Observable<List<ImageAlbumEntity>> =
                Observable.fromArray(
                        store.raw(
                                ImageAlbumEntity::class,
                                "SELECT alb.* " +
                                "from ImageAlbum alb " +
                                "    left join ImageAlbumJunction albJun on albJun.albumId = alb.id " +
                                "WHERE albJun.parentId is $parentId"
                        ).toList()
                )

        override fun getAlbum(albumId: Int): Observable<ImageAlbumEntity> =
                store.select(ImageAlbumEntity::class)
                        .where(ImageAlbumEntity.ID.eq(albumId))
                        .get().observable()

        override fun getEstate(estateId: Int): Observable<EstateEntity> =
                store.select(EstateEntity::class)
                        .where(EstateEntity.ID.eq(estateId))
                        .get().observable()

        override fun getPhotos(albumId: Int): Observable<GalleryPhotoEntity> =
                store.select(GalleryPhotoEntity::class)
                        .where(GalleryPhotoEntity.ALBUM_ID.eq(albumId))
                        .get().observable()

        override fun getPhoto(photoId: Int): Observable<GalleryPhotoEntity> =
                store.select(GalleryPhotoEntity::class)
                        .where(GalleryPhotoEntity.ID.eq(photoId))
                        .get().observable()

        override fun fetchArticles(feedAlias: String, pageSize: Int, startIndex: Int)
                : Observable<ArticleEntity> =
                fetchAll<ArticleEntity>()
                        .where(ArticleEntity.FEED_ID.eq(feedAlias))
                        .orderBy(ArticleEntity.PUBLISHED.desc())
                        .limit(pageSize)
                        .offset(startIndex)
                        .get().observable()

        override fun checkIfExistsAlbumJunction(albumId: Int, parentAlbumId: Int) =
                fetchAll<ImageAlbumJunctionEntity>()
                        .where(ImageAlbumJunctionEntity.ALBUM_ID.eq(albumId))
                        .and(ImageAlbumJunctionEntity.PARENT_ID.eq(parentAlbumId))
                        .get()
                        .firstOrNull()

        override fun fetchUser(uuid: Int): Observable<UserEntity> =
                store.select(UserEntity::class).where(UserEntity.UUID.eq(uuid)).get().observable()
//                fetchAll<UserEntity>().where(UserEntity.UUID.eq(uuid)).get().observable()


        override fun fetchAllUsers(): Observable<List<UserEntity>> =
                store.select(UserEntity::class).get().observable().toList().toObservable()

        fun upsertFeeds(entity: List<FeedEntity>) =
                store.upsert(entity)

        override fun userObservableResult(uuid: Int): Observable<ReactiveResult<UserEntity>> =
                store.select(UserEntity::class)
                        .where(UserEntity.UUID.eq(uuid))
                        .get()
                        .observableResult()

        override fun getUserFavorites(uuid: Int): Observable<EstateEntity> =
                store.select(EstateEntity::class)
                        .where(EstateEntity.USER_ID.eq(uuid))
                        .and(EstateEntity.FAVORITE.eq(true))
                        .get().observable()

        override fun setArticleReadState(id:Int,state:Boolean): Completable =
                store.update(ArticleEntity::class)
                        .set(ArticleEntity.READ,state)
                        .where(ArticleEntity.ID.eq(id))
                        .get()
                        .single()
                        .ignoreElement()


        override fun setDealReadState(id:String,state:Boolean): Completable =
                store.update(DealEntity::class)
                        .set(DealEntity.READ,state)
                        .where(DealEntity.ID.eq(id))
                        .get()
                        .single()
                        .ignoreElement()

        override fun saveSubscription(subscription:SubscriptionEntity)
                : Observable<SubscriptionEntity> = store.upsert(subscription)
                        .toObservable()
                        .doOnNext { store.refresh(subscription.owner).blockingGet() }


        override fun getSubscription(id:Int): ReactiveResult<SubscriptionEntity> =
                store.select(SubscriptionEntity::class)
                        .where(SubscriptionEntity.ID.eq(id))
                        .get()





        ///region tours files


        override fun getTourPreviewsObservableResult(estateId: Int) = store
                .select(TourPreviewEntity::class)
//            .join(UserEntity::class).on(UserEntity::uuid.eq(estateId))
                .where(TourPreviewEntity.ESTATE_ID.eq(estateId))
                .get()
                .observableResult()

        override fun getTourPreview(tourId: String) = store
                .select(TourPreviewEntity::class)
                .where(TourPreviewEntity.ID.eq(tourId))
                .get()

        override fun getEstateTourPreviews(estateId: Int): Observable<List<TourPreviewEntity>> = store
                .select(TourPreviewEntity::class)
                .where(TourPreviewEntity.ESTATE_ID.eq(estateId))
                .get()
                .observable()
                .toList()
                .toObservable()
                .map { it.toList() }

        override fun upsertTourPreview(list: List<TourPreviewEntity>) = store
                .upsert(list)
                .doOnSuccess { Timber.d("Upsertd succses %s", it.count()) }
                .toObservable()

        override fun upsertTourPreview(entity: TourPreviewEntity) = store
                .upsert(entity)
                .toObservable()


        override fun upsertTourFileJunction(entity: List<TourFileJunctionEntity>)
                : Observable<Iterable<TourFileJunctionEntity>> = store.upsert(entity).toObservable()



        override fun upsertTourFileJunction(entity: TourFileJunctionEntity) = store
                .upsert(entity)
                .toObservable()

        override fun observableTourDownloadState() = store
                .select(TourPreviewEntity::class)
                .where(TourPreviewEntity.DOWNLOADED.notIn(arrayListOf(DownloadState.NotDownloaded, DownloadState.MetaPreparation)))
                .get()
//            .flowable()
//            .
                .observableResult()
                .flatMap { it.observable() }
                .toFlowable(BackpressureStrategy.BUFFER)
                .subscribeOn(Schedulers.io())


        override fun getFileEntitysOrDefault(uri: RevisionString) = store
                .select(FileEntity::class)
                .where(FileEntity.URI.eq(uri))
                .get()
                .firstOr {
                        val entity = FileEntity()
                        entity.setUri(uri)
                        entity.setDownloadedSize(0)
                        entity.setTotalSize(0)
                        entity.setDownloaded(false)
                        entity
                }


        override fun getFileEntity(uri: RevisionString) = store
                .select(FileEntity::class)
                .where(FileEntity.URI.eq(uri))
                .get()


        override fun getTourFiles(entity : TourPreviewEntity ) =store
                .select(TourFileJunctionEntity::class)
                .where(TourFileJunctionEntity.TOUR.eq(entity.id))
                .get()
                .toList()

        override fun getTourFiles(tourId : String) =store
                .select(TourFileJunctionEntity::class)
                .where(TourFileJunctionEntity.TOUR.eq(tourId))
                .get()
                .toList()


        override fun getTourFilesObservable(tourId : String) =store
                .select(TourFileJunctionEntity::class)
                .where(TourFileJunctionEntity.TOUR.eq(tourId))
                .get()


        override fun pushFileEntities(uris: List<FileEntity>) = store
                .select(FileEntity::class)
                .where(FileEntity.URI.`in`(uris.map { it.uri }.toList()))
                .get()
                .observable()
                .toList()
                .map {existedUris-> uris.filter{!existedUris.map {file -> file.uri.revisionUri() }.contains(it.uri.revisionUri()) }.toList()}
                .flatMapObservable{ store.insert(it).toObservable() }



        override fun fetchFileEntityes(uris: List<RevisionString>) = store
                .select(FileEntity::class)
                .where(FileEntity.URI.`in`(uris))
                .get()
                .toList()


        override fun fetchTouresWithStates(states: List<DownloadState>) = store
                .select(TourPreviewEntity::class)
                .where(TourPreviewEntity.DOWNLOADED.`in`(states))
                .get()
                .toList()


        override fun getDownloadedSumFileEntityes(uris: List<RevisionString>) = store
                .select(FileEntity::class)
                .where(FileEntity.URI.`in`(uris))
                .get()
                .map{ it.downloadedSize }
                .sum()
//            .observable()


        override fun flowableFileEntityes(uris: List<RevisionString>) = store
                .select(FileEntity::class)
                .where(FileEntity.URI.`in`(uris))
                .get()
                .flowable()
                .subscribeOn(Schedulers.io())
                .observeOn(Schedulers.computation())


        override fun upsertFileEntity(entity: FileEntity) = store
                .upsert(entity)
                .toObservable()


        override fun upsertFileEntity(entity: List<FileEntity>) = store
                .upsert(entity)
                .toObservable()


        override fun getTourFilesJunction(tourId:String) = store
                .select(TourFileJunctionEntity::class)
                .where(TourFileJunctionEntity.TOUR.eq(tourId))
                .get()

        override fun getTourFilesJunctionUniqueFiles(tourId:String):
                List<TourFileJunctionEntity?> =
                store
                        .raw(TourFileJunctionEntity::class,
                                "SELECT tfj.* from TourFileJunction tfj " +
                                        "join TourFileJunction tfj2 on tfj2.file = tfj.File " +
                                        "WHERE tfj2.tour = $tourId " +
                                        "GROUP BY tfj.file " +
                                        "HAVING COUNT(tfj.file) < 2"
                        )
                        .toList()


        private fun <E:Persistable> deleteBlocking(entity:E) = store
                .delete(entity)
                .blockingAwait()

        private fun <E:List<Persistable>> deleteBlocking(entity:E) = store
                .delete(entity)
                .blockingAwait()

        override fun deleteTourFilesJunction(entity: List<TourFileJunctionEntity>) =
                deleteBlocking(entity)

        override fun deleteTourFilesJunction(tourId:String) = store
                .delete(TourFileJunctionEntity::class)
                .where(TourFileJunctionEntity.TOUR.eq(tourId))
                .get()

        override fun deleteFile(entity:FileEntity) = deleteBlocking(entity)

        override fun deleteFiles(entity:List<FileEntity>) = deleteBlocking(entity)

        override fun deleteTourPreview(entity: TourPreviewEntity) = deleteBlocking(entity)


        override fun deleteTourPreview(id:String) = store
                .delete(TourPreview::class)
                .where(TourPreviewEntity.ID.eq(id))
                .get()
                .single()
                .blockingGet()
        ///endregion

}
