package com.biganto.visual.roompark.domain.use_case


import android.app.Application
import android.media.MediaScannerConnection
import com.biganto.visual.roompark.data.repository.api.biganto.IBigantoApi
import com.biganto.visual.roompark.data.repository.db.IDb
import com.biganto.visual.roompark.data.repository.db.requrey.RevisionString
import com.biganto.visual.roompark.data.repository.db.requrey.model.FileEntity
import com.biganto.visual.roompark.data.repository.db.requrey.model.TourFileJunctionEntity
import com.biganto.visual.roompark.data.repository.db.requrey.model.fromRaw
import com.biganto.visual.roompark.data.repository.file.FileModule
import com.biganto.visual.roomparkvr.data.repository.db.requery.model.DownloadState
import com.biganto.visual.roomparkvr.data.repository.db.requery.model.TourPreviewEntity
import io.reactivex.BackpressureStrategy
import io.reactivex.Flowable
import io.reactivex.Observable
import io.reactivex.schedulers.Schedulers
import okhttp3.ResponseBody
import okio.Okio
import timber.log.Timber
import java.io.File
import java.util.*
import java.util.concurrent.TimeUnit
import javax.inject.Inject

/**
 * Created by Vladislav Bogdashkin on 02.08.2019.
 */

private const val META_PREDICTION="/tourMeta_"
private const val META_FILE_TYPE=".json"

class DownloadUseCase @Inject constructor(
    private val db: IDb,
    private val api: IBigantoApi,
    private val fileModule: FileModule,
    private val context: Application
) {

    private fun writeFile(
        response: ResponseBody,
        model: TourFileData,
        token: CancellationToken
    ): Observable<TourFileData> {
        return Observable.create<TourFileData> { sub ->
            try {
                if (token.isCancelled) {
                    sub.onNext(model)
                    sub.onComplete()
                }

                val fileStorage = fileModule.getFile(model.fileUrl.uri())
                val sink =
                    if (model.fileDownloadedSize>0)
                        Okio.buffer(Okio.appendingSink(fileStorage))
                    else  Okio.buffer(Okio.sink(fileStorage))
                val buffer = sink.buffer()
                var read = 0L
                val step = 4096
                val source = response.source()
                var stop: Boolean = token.isCancelled
                sink.use {
                    while (!stop && { read = source.read(buffer, step.toLong());read }() != -1L) {
                        model.tempDownloadedSize += read
                        model.fileDownloadedSize += read

                        if (model.tempOverallFileSize == 0L)
                            model.tempTourTotalDiff += model.tempDownloadedSize

                        if (token.isCancelled) {
                            model.isDownloaded = false
                            model.fileDownloadedSize = 0
                            sub.onComplete()
                            stop = true
                            source.buffer.flush()
                        }
                    }
                }

                model.isDownloaded = (source.exhausted()
                        && (model.fileDownloadedSize == model.tempOverallFileSize
                        || model.tempOverallFileSize == 0L))

                sub.onNext(model.copy())
                model.tempTourTotalDiff = 0
                model.tempDownloadedSize = 0
                sub.onComplete()
                sink.close()
            } catch (e: Throwable) {
                Timber.e(e)
                if (!sub.isDisposed)
                    sub.onError(e)
                token.isCancelled = true
            }
        }
    }


    private fun mergeFiles(files: List<FileEntity>) {
        Timber.d("Nerge files")
        files.forEach { file ->
            val entity: FileEntity? = db.getFileEntity(file.uri).firstOrNull()
            entity?.let {
                file.setDownloaded(it.isDownloaded)
                file.setDownloadedSize(it.downloadedSize)
                file.setTotalSize(it.totalSize)
            }
        }

        Timber.d("save files: ${files.size}")
        db.upsertFileEntity(files)?.blockingSubscribe { Timber.d("file saved") }
    }

    @Volatile
    private var tourDbModel: TourPreviewEntity? = null

    private fun setDownloadInfo(
        id: String,
        downloadedSize: Long? = null
        , downloadedDiffSize: Long? = null
        , totalSize: Long? = null
        , resolution: Int? = null
        , filesCount: Int? = null
        , tempLoadedFiles: Int? = null
        , totalSizedDiffSize: Long? = null
    ) {
        if (tourDbModel?.id != id)
            throw error("Wrong tour id")
        tourDbModel?.let { entity ->
            downloadedSize?.let { entity.downloadedSize = it }
            downloadedDiffSize?.let { entity.downloadedSize += it }
            totalSize?.let { entity.overallSize = it }
            resolution?.let { entity.targetResolution = it }
            filesCount?.let { entity.overallFiles = it }
            tempLoadedFiles?.let { entity.downloadedFiles += it }
            totalSizedDiffSize?.let { entity.overallSize += it }
            if (entity.downloadedFiles == entity.overallFiles)
                entity.isDownloaded = DownloadState.Downloaded
        }
        Timber.d(" tour: ${tourDbModel?.downloadedFiles} / ${tourDbModel?.overallFiles}")
    }


    private fun observableTourDownloading(tour: TourPreviewEntity, token: CancellationToken) =
        api.getTourFiles(tour.id, tour.targetResolution.toString())
            .map { tourDbModel = tour;it.first() }
            .map { raw ->
                var downloadedSize = 0L
                var totalSize = 0L
                val fileEntities = raw.files.map(::fromRaw)

                mergeFiles(fileEntities)
                val jlist = db.getTourFilesJunction(tour.id).toList()
                val junctionList = fileEntities
                    .map {file ->
                        val entity = jlist.firstOrNull{it.tour == tour.id && it.file == file.uri}
                            ?: TourFileJunctionEntity()
                        entity.setTour(tour.id)
                        entity.setFile(file.uri)
                        downloadedSize += file.downloadedSize
                        totalSize += file.totalSize
                        entity
                    }
                setDownloadInfo(
                    raw.id.toString()
                    , downloadedSize = downloadedSize
                    , totalSize = totalSize
                    , resolution = raw.resolution
                    , filesCount = raw.files.count()
                )

                junctionList
            }
            .doOnNext { junctionList ->
                db.upsertTourFileJunction(junctionList)?.subscribe { Timber.d("junction upserted") }
            }
            .doOnNext { _ ->
                tourDbModel?.let {
                    db.upsertTourPreview(it)?.subscribe { Timber.d("tour upserted") }
                }
            }
            .flatMapIterable { it }
            .flatMap { junction ->
                db.getFileEntity(junction.file)
                    .observable()
                    .map { entity ->
                        TourFileData(
                            fileUrl = junction.file
                            , tourId = junction.tour
                            , tempDownloadedSize = 0L
                            , tempOverallFileSize = entity.totalSize
                            , fileDownloadedSize = entity.downloadedSize
                            , tempTourTotalDiff = 0L
                            , isDownloaded = entity.isDownloaded
                        )
                    }

            }
            .toFlowable(BackpressureStrategy.BUFFER)
            .parallel(4)
            .runOn(Schedulers.io())
            .filter { !token.isCancelled }
            .flatMap { model ->
                if (model.isDownloaded)
                    return@flatMap Flowable.just(model)

                var header: HashMap<String, String>? = null
                if (model.fileDownloadedSize > 0){
                    header = hashMapOf(Pair("Range", "bytes=${model.fileDownloadedSize}-"))
                    Timber.w("trying to continue download file " +
                            "url by: ${model.fileUrl}" +
                            "size is: ${model.fileDownloadedSize}/${model.tempOverallFileSize}" +
                            "and header is: $header")
                }


                api.downloadFile(model.fileUrl.revisionUri(), header)
                    .doOnError {
                        Timber.e(it)
                    }
                    .flatMap<TourFileData> {
                        writeFile(it, model, token)
                            .toFlowable(BackpressureStrategy.BUFFER)
                            .doOnCancel { Timber.d("CANCELLED") }
                    }
                    .flatMap { downloadInfo ->
                        db.upsertFileEntity(
                            FileEntity().also{
                                it.setUri(downloadInfo.fileUrl)
                                it.setDownloadedSize(downloadInfo.fileDownloadedSize)
                                it.setTotalSize(downloadInfo.tempOverallFileSize)
                                it.setDownloaded(downloadInfo.isDownloaded)})
                            .toFlowable(BackpressureStrategy.BUFFER)
                            .map { downloadInfo }

                    }
            }
            .sequential()
            .toObservable()
//            .buffer(15L,TimeUnit.MILLISECONDS)
//            .flatMapIterable { it }
            .map { model ->
                setDownloadInfo(
                    model.tourId
                    , totalSizedDiffSize = model.tempTourTotalDiff
                    , downloadedDiffSize = model.tempDownloadedSize
                    , tempLoadedFiles = if (model.isDownloaded) 1 else null
                )
                model.tempDownloadedSize = 0
                model.tourId
            }
            .delay(14L, TimeUnit.MILLISECONDS)
//            .sample(37L, TimeUnit.MILLISECONDS)
            .flatMap { db.upsertTourPreview(tourDbModel!!) }


    fun startTourDownloading(tourId: String, cancellataionToken: CancellationToken)
            : Observable<TourPreviewEntity> = db.getTourPreview(tourId).observable()
        .doOnNext {
            it.isDownloaded = DownloadState.Downloading
            it.downloadedFiles = 0
            it.downloadedSize = 0L
            it.tempSize = it.overallSize  // <- overall changes due downloading!!
            it.overallSize = 0L

        }
        .flatMap { db.upsertTourPreview(it) }
        .flatMap(::getMeta)
        .doOnError(Timber::e)
        .flatMap { observableTourDownloading(it, cancellataionToken) }


    private fun getMeta(tour: TourPreviewEntity) =
        api.getTourMetaAsString(tour.id)
            ?.doOnNext { meta ->
                tour.let {
                    val metaUri = RevisionString("$META_PREDICTION${tour.id}$META_FILE_TYPE")
                    it.setMetaFileEntityId(metaUri)
                    fileModule.saveFileToDisk(
                        File(
                        FileModule.assetsDirectory(context).plus(metaUri.uri())
                        ),meta
                    )
                }
            }
            ?.map { tour }
            ?.onErrorReturn {
                tour.isDownloaded = DownloadState.Crushed
                db.upsertTourPreview(tour)?.blockingSubscribe()
                tour
            }

    //#endregion oldMethod


    private fun refreshGallery(file: File) {
        MediaScannerConnection.scanFile(
            context, arrayOf(file.path), null
        )
        { path, uri ->
            {}//Timber.d("Scanned $path")
        }
    }

    data class TourFileData(
        val fileUrl: RevisionString,
        val tourId: String,
        var tempDownloadedSize: Long = 0L,
        var tempOverallFileSize: Long = 0L,
        var fileDownloadedSize: Long = 0L,
        var tempTourTotalDiff: Long = 0L,
        var isDownloaded: Boolean = false
    )

    data class CancellationToken(var isCancelled: Boolean)
}