package com.biganto.visual.roompark.domain.use_case

import android.content.res.Resources
import com.biganto.visual.roompark.data.repository.api.biganto.IBigantoApi
import com.biganto.visual.roompark.data.repository.api.biganto.raw.TourPreviewRaw
import com.biganto.visual.roompark.data.repository.db.IDb
import com.biganto.visual.roompark.domain.contract.AuthContract
import com.biganto.visual.roompark.domain.custom_exception.CustomApiException
import com.biganto.visual.roompark.domain.model.TourModel
import com.biganto.visual.roompark.domain.model.fromEntity
import com.biganto.visual.roomparkvr.data.repository.db.requery.model.DownloadState
import com.biganto.visual.roomparkvr.data.repository.db.requery.model.TourPreviewEntity
import com.biganto.visual.roomparkvr.data.repository.db.requery.model.fromRaw
import io.reactivex.Observable
import io.reactivex.schedulers.Schedulers
import timber.log.Timber
import javax.inject.Inject

/**
 * Created by Vladislav Bogdashkin on 19.06.2018.
 */

const val EMPTY_PARENT = -778

class TourPreviewsUseCase @Inject constructor(
    private val api:IBigantoApi
    ,private val db: IDb
    ,private val auth:AuthContract
) {

    fun getTourById(tourId:String):Observable<DownloadTourByIdResult> {
        val tour = retrieveTour(tourId)
        Timber.d("retrieveTour $tourId / $tour")
        if (tour != null) {
            if (tour.downloadState == DownloadState.Downloaded)
                return Observable.just<DownloadTourByIdResult>(DownloadTourByIdResult.ReadyToPlay(tour))

            return setTourState(tour.tour_id, DownloadState.DownloadQueue)
                .map<DownloadTourByIdResult> { DownloadTourByIdResult.DownloadStarted(tour) }
        }

        return syncTour(tourId, parentId = EMPTY_PARENT)
            .map { it }
            .map<DownloadTourByIdResult> { DownloadTourByIdResult.DownloadStarted(it) }
    }

    fun retrieveTour(tourId: String) =
            db.getTourPreview(tourId)
                .asSequence()
                .map ( ::fromEntity)
                .firstOrNull()

    private fun setTourState(tourId:String, state: DownloadState): Observable<Boolean> =
                    db.getTourPreview(tourId)
                        .observable()
                        .doOnNext{ it.isDownloaded=state }
                        .flatMap { db.upsertTourPreview(it)?.map{true} }

    private fun syncTour(tourId:String, parentId:Int):Observable<TourModel> =
        syncTour(tourId,parentId,calcTargetResolution())

    private fun syncTour(tourId:String, estateId: Int, targetResolution:Int)
            : Observable<TourModel> {
        val request = ToursRemoteByIdRequestModel(
            arrayListOf(tourId)
            , estateId
            , null
            , targetResolution
        )
        return forceTourUpdate(request)
            .map {
                if (it.isNullOrEmpty())
                    throw CustomApiException.TourByIdNotFoundException()
                it.first() }
            .doOnNext { it.isDownloaded = DownloadState.DownloadQueue }
            .flatMap {db.upsertTourPreview(it)}
            .map(::fromEntity)
            .doOnError(Timber::e)
    }


    private fun calcTargetResolution():Int{
        val w = Resources.getSystem().displayMetrics.widthPixels
        val h = Resources.getSystem().displayMetrics.heightPixels
        return w.coerceAtLeast(h)
    }

    private fun forceTourUpdate(requestModel: ToursRemoteByIdRequestModel): Observable<List<TourPreviewEntity>> =
        api.getToursPreviewById(requestModel.ids)
            .subscribeOn(Schedulers.io())
            .map { fromRaw(it
                , requestModel.estateId
                , api.provideHttpUrl().toString()
                , requestModel.targetResolution)
            }


    private fun fetchDbTourList(estateId: Int) =
        db.getEstateTourPreviews(estateId)
            .filter { !it.isNullOrEmpty() }

    private fun fetchApiTourList(multitourId:Int,estateId: Int) =
        auth.currentUser()
            .doOnNext { Timber.d("user on next: $it") }
            .map { it.targetResolution }
            .flatMap { res ->
            Timber.d("user: $res")
            api.getOfferTours(multitourId)
                .doOnError { Timber.e(it) }
                .map {
                    mergeRaw(it, TourRemoteRequestModel(
                            estateId,
                            api.provideHttpUrl().toString(),
                            res
                        )
                    )
                }
                .map { it }
                .doOnNext { db.blockingUpsert(it) }
        }

    fun fetchTourOffer(multitourId:Int,parent:Int) =
        Observable.mergeDelayError(
            arrayListOf(fetchDbTourList(parent),fetchApiTourList(multitourId,parent))
        )
            .take(1)
            .doOnError { Timber.e(it) }
            .map (::fromEntity)
            .subscribeOn(Schedulers.io())


    private fun mergeRaw(raw: List<TourPreviewRaw>, requestRequestModel: TourRemoteRequestModel) =
        raw.map{ mergeRaw(it, requestRequestModel) }

    private fun mergeRaw(raw: TourPreviewRaw, requestRequestModel: TourRemoteRequestModel)
            : TourPreviewEntity {
        Timber.d("got raw: $raw")
        val entity = db.getTourPreview(raw.id.toString()).firstOrNull()
            ?: return fromRaw(raw
                , requestRequestModel.estateId
                , api.provideHttpUrl().toString()
                , requestRequestModel.targetResolution)


        val fromApi = fromRaw(raw
            , requestRequestModel.estateId
            , api.provideHttpUrl().toString()
            , requestRequestModel.targetResolution)

        if (fromApi.updated > entity.updated
            && entity.isDownloaded == DownloadState.Downloaded) {
            entity.isDownloaded =
                DownloadState.NotSynced//maxState(entity.isDownloaded, fromApi.isDownloaded)
            return entity
        }

        // In case< where we need to update tour parent, when it was downloaded by link
//        if (fromApi.estate.id!=requestRequestModel.estateId)
//            entity.estate=fromApi.estate
        fromApi.setMetaFileEntityId(entity.metaFileEntityId)
        fromApi.targetResolution=entity.targetResolution
        fromApi.isDownloaded=entity.isDownloaded
        fromApi.overallSize=entity.overallSize
        fromApi.downloadedSize=entity.downloadedSize
        fromApi.tempSize=entity.tempSize
        fromApi.overallFiles=entity.overallFiles
        fromApi.downloadedFiles=entity.downloadedFiles

        return fromApi
    }
}

data class TourRemoteRequestModel(val estateId: Int, val token: String? = null, val targetResolution: Int)


data class ToursRemoteByIdRequestModel(val ids: List<String>, val estateId: Int,
                                       val token: String? = null,  val targetResolution: Int)


sealed class DownloadTourByIdResult{
    class ReadyToPlay(val tour:TourModel): DownloadTourByIdResult()
    class DownloadStarted(val tour:TourModel): DownloadTourByIdResult()
    class TourError(val id:String): DownloadTourByIdResult()

}
