package com.biganto.visual.roompark.presentation.screen.albums

import com.biganto.visual.roompark.conductor.BigantoBasePresenter
import com.biganto.visual.roompark.domain.interactor.AlbumsInteractor
import com.biganto.visual.roompark.domain.model.AlbumPreviewModel
import com.biganto.visual.roompark.domain.model.AlbumSortedModel
import com.biganto.visual.roompark.domain.model.SubscriptionModel
import com.biganto.visual.roompark.domain.model.SubscriptionTopic
import com.biganto.visual.roompark.util.monades.ExceptionString
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import timber.log.Timber
import javax.inject.Inject
import javax.inject.Named

/**
 * Created by Vladislav Bogdashkin on 30.09.2019.
 */

class AlbumsScreenPresenter @Inject constructor(
    private val interactor: AlbumsInteractor
    ,@Named(SELECTED_ALBUM_INDEX_KEY) private var selectedIndex:Int
)
    : BigantoBasePresenter<AlbumsScreen, AlbumsScreenViewState>() {

    private var restoreModel = RestoreModel(listOf(), listOf(), selectedIndex,null)

    override fun defaultErrorViewStateHandler() =
        { e: ExceptionString -> AlbumsScreenViewState.SomeError(e) }

    private fun requestAlbum(id: Int): Observable<AlbumsScreenViewState> =
        interactor.fetchAlbumPhotos(id)
            .filter { !it.isNullOrEmpty() }
            .doOnNext { restoreModel.albumList = it }
            .map { AlbumsScreenViewState.AlbumsSelected(it) }


    private fun fetchSubscription(albumId: Int): Observable<AlbumsScreenViewState> =
        interactor.getSubscriptions(currentAlbumTopic(albumId))
        .doAfterNext { restoreModel.sub = it }
        .map<AlbumsScreenViewState> { AlbumsScreenViewState.SubscriptionStatus(it.state) }
//            .startWith(Observable.just<AlbumsScreenViewState>(AlbumsScreenViewState.Idle()))

    override fun bindIntents() {

        val onSubChecked = intent(AlbumsScreen::onSubscription)
            .filter { restoreModel.sub != null }
            .flatMap { newState ->
                interactor.switchSubscription(restoreModel.sub!!, newState)
                    .andThen(
                        Observable.just<AlbumsScreenViewState>(
                            AlbumsScreenViewState.SubscriptionStatus(
                                newState
                            )
                        )
                    )
                    .onErrorReturn { AlbumsScreenViewState.SubscriptionError(!newState) }
            }


        val fetchParents = interactor.fetchHeaderAlbums()
            .filter { !it.isNullOrEmpty() }
            .doOnNext { restoreModel.albumsPreview = it }
            .map { AlbumsScreenViewState.AlbumsListLoaded(it, selectedIndex) }

        val fetchSelected = requestAlbum(selectedIndex)

        val headerItemSelected = intent(AlbumsScreen::onAlbumSelected)
            .doOnNext { selectedIndex = it.albumId }
            .doOnNext { restoreModel.currentIndex = it.albumId }
            .switchMap<AlbumsScreenViewState> { model ->
                requestAlbum(model.albumId)
                    .onErrorReturn(::parseError)
                    .startWith(Observable.just<AlbumsScreenViewState>(AlbumsScreenViewState.HeaderAlbumChoosed(item = model))
                    )
            }

        val fetchHeaderSubscription = intent(AlbumsScreen::onAlbumSelected)
            .map { it.albumId }
            .doOnNext { selectedIndex = it ; restoreModel.currentIndex = it }
            .switchMap { fetchSubscription(it) }


        val photoSelected = intent(AlbumsScreen::onPhotoSelected)
            .map { AlbumsScreenViewState.PhotoSelected(it.photoId) }


        val state = Observable.mergeDelayError(
            arrayListOf(
                fetchParents,
                fetchSelected,
                headerItemSelected,
                photoSelected,
                fetchSubscription(selectedIndex),
                fetchHeaderSubscription,
                onSubChecked
            )
        )
            .doOnError { Timber.e(it) }
            .onErrorReturn(::parseError)
            .subscribeOn(Schedulers.io())
            .observeOn(AndroidSchedulers.mainThread())
        subscribeViewState(state.cast(AlbumsScreenViewState::class.java), AlbumsScreen::render)
    }

    override fun detachView() {
        super.detachView()
        restoreStateObservable.accept(AlbumsScreenViewState.RestoreView(restoreModel))
    }

    private fun currentAlbumTopic(albumId:Int):SubscriptionTopic  {
        val z =  when(albumId){
            9 -> SubscriptionTopic.Progress_2()
            10 -> SubscriptionTopic.Progress_1()
            11 -> SubscriptionTopic.Progress_3()
            106 -> SubscriptionTopic.Progress_kindergarden()
            else -> error("unknown parent album type!")
        }
        return z
    }
}

data class RestoreModel(var albumList:List<AlbumSortedModel>
                        ,var albumsPreview:List<AlbumPreviewModel>
                        , var currentIndex:Int
                        , var sub: SubscriptionModel?)