package com.biganto.visual.roompark.presentation.screen.estate

import android.content.Context
import androidx.annotation.StringRes
import com.biganto.visual.roompark.R
import com.biganto.visual.roompark.conductor.BigantoBasePresenter
import com.biganto.visual.roompark.domain.interactor.EstateInteractor
import com.biganto.visual.roompark.domain.interactor.ToursInteractor
import com.biganto.visual.roompark.domain.model.*
import com.biganto.visual.roompark.presentation.screen.estate.util.DisplayInfoModel
import com.biganto.visual.roompark.util.extensions.toRubly
import com.biganto.visual.roompark.util.monades.ExceptionString
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import timber.log.Timber
import javax.inject.Inject
import javax.inject.Named

/**
 * Created by Vladislav Bogdashkin on 30.09.2019.
 */


class EstateScreenPresenter @Inject constructor(
    private val interactor: EstateInteractor,
    private val toursInteractor: ToursInteractor,
    private val context: Context,
    @Named(SELECTED_ESTATE_ID_KEY) private val estateId:Int
)
    : BigantoBasePresenter<EstateScreen, EstateScreenViewState>() {

    private var planList: List<PlanPresetModel>? = null

    private var estate: EstateModel? = null

    private var showType: InfoShowType = InfoShowType.COMMON_INFO

    override fun defaultErrorViewStateHandler() =
        { e: ExceptionString -> EstateScreenViewState.SomeError(e) }

    private fun getPlan(plan: PlanPresetModel): Observable<EstateScreenViewState> =
        interactor.getPlan(plan)
            .map<EstateScreenViewState> { EstateScreenViewState.LoadPlan(it) }

    override fun bindIntents() {

        val prefetchCards = interactor.getEstate(estateId)
            .doOnNext { estate = it.copy() }
            .map { EstateScreenViewState.LoadEstate(it) }

        val fetchPlans = interactor.getPlanTypes(estateId)
            .doOnNext { planList = it.toList() }
            .map { EstateScreenViewState.LoadPlanTypes(it) }

        val fetchPlan = intent(EstateScreen::planTypesTabSelected)
            .map { planList?.get(it) }
            .flatMap { planPreset ->
                interactor.getPlan(planPreset)
                    .map<EstateScreenViewState> { plan -> EstateScreenViewState.LoadPlan(plan) }
                    .startWith(
                        Observable.just<EstateScreenViewState>(
                            EstateScreenViewState.PlanTypeSelected(planPreset)
                        )
                    )
                    .startWith(
                        Observable.just<EstateScreenViewState>(

                            EstateScreenViewState.ShowEstateInfo(
                                showType
                                , if (showType == InfoShowType.COMMON_INFO)
                                    mapCommonInfo(estate?.commonInfo)
                                else mapCommonInfo(planPreset.explication)
                            )
                        )
                    )
            }

        val switchSizes = intent(EstateScreen::switchSizes)
            .map { pair ->
                val plan = planList?.first { it.planId == pair.first }
                plan?.switchFeature<FeatureModel.Sizes>(pair.second)
                plan
            }.flatMap(::getPlan)

        val switchFurn = intent(EstateScreen::switchFurniture)
            .map { pair ->
                val plan = planList?.first { it.planId == pair.first }
                plan?.switchFeature<FeatureModel.Furniture>(pair.second)
                plan
            }.flatMap(::getPlan)

        val switchWalls = intent(EstateScreen::switchWalls)
            .map { pair ->
                val plan = planList?.first { it.planId == pair.first }
                plan?.switchFeature<FeatureModel.Walls>(pair.second)
                plan
            }.flatMap(::getPlan)

        val switchElectric = intent(EstateScreen::switchElectric)
            .map { pair ->
                val plan = planList?.first { it.planId == pair.first }
                plan?.switchFeature<FeatureModel.Electric>(pair.second)
                plan
            }.flatMap(::getPlan)


        val showInfo = intent(EstateScreen::showCommonInfo)
            .doOnNext { showType = InfoShowType.COMMON_INFO }
            .map { estate?.commonInfo }
            .map(::mapCommonInfo)
            .map { EstateScreenViewState.ShowEstateInfo(showType, it) }

        val showExplication = intent(EstateScreen::showExplication)
            .doOnNext { showType = InfoShowType.EXPLICATIONS }
            .map { planList?.get(it)?.explication }
            .map(::mapCommonInfo)
            .map { EstateScreenViewState.ShowEstateInfo(showType, it) }


        val onStartTours = intent(EstateScreen::tourCardClicked)
            .map { estate }
            .flatMap {estate -> toursInteractor.getEstateTourList(estate)
                .map { EstateScreenViewState.ToursLoaded(it) }
            }


        val state = Observable.mergeDelayError(
            arrayListOf(
                restoreStateObservable,
                prefetchCards,
                fetchPlans,
                fetchPlan,
                switchElectric,
                switchFurn,
                switchSizes,
                switchWalls,
                showInfo,
                onStartTours,
                showExplication
            )
        )
            .doOnError { Timber.e(it) }
            .subscribeOn(Schedulers.io())
            .observeOn(AndroidSchedulers.mainThread())

        subscribeViewState(state.cast(EstateScreenViewState::class.java), EstateScreen::render)
    }

    private fun langString(@StringRes id:Int) = context.resources.getString(id)

    private fun langString(@StringRes id:Int,vararg args:Any) =
            context.resources.getString(id,*args)


    private fun mapCommonInfo(list: List<ExplicationModel>): ArrayList<DisplayInfoModel> =
        ArrayList(
            List(list.size){Timber.d("area value: ${list[it].area}");DisplayInfoModel(list[it].title
                ,langString(R.string.area_value,list[it].area))
            }
        )

    private fun mapCommonInfo(info: CommonInfoModel?): ArrayList<DisplayInfoModel> {
        val r = arrayListOf<DisplayInfoModel>()
        if (info == null) return r
        info.building.let {
            r.add(DisplayInfoModel(langString(R.string.building),"$it"))
        }
        info.section_begin.let {
            r.add(DisplayInfoModel(langString(R.string.section_begin),"$it"))
        }
        r.add(DisplayInfoModel(langString(R.string.floor),"${info.floor}/${info.floor_max}"))
        info.area?.let {
            r.add(DisplayInfoModel(langString(R.string.area),langString(R.string.area_value,it)))
        }
        info.area_living?.let {
            r.add(DisplayInfoModel(
                langString(R.string.area_living),langString(R.string.area_value,it))
            )
        }
        info.rooms?.let {r.add(DisplayInfoModel(langString(R.string.rooms),"$it"))}
        info.kind?.let {r.add(DisplayInfoModel(langString(R.string.flat_kind), it))}
        info.decoration?.let {r.add(DisplayInfoModel(langString(R.string.flat_decoration), it))}
        info.ceiling?.let {r.add(DisplayInfoModel(langString(R.string.ceiling)
            , langString(R.string.meters_value,it)))
        }
        info.windows_face?.let {r.add(DisplayInfoModel(langString(R.string.window_face), it))}
        info.direction?.let {r.add(DisplayInfoModel(langString(R.string.direction), it))}
        info.price_meter?.let {r.add(DisplayInfoModel(langString(R.string.price_meter), it
            .toRubly()))}
        info.price?.let {r.add(DisplayInfoModel(langString(R.string.price), it.toRubly()))}
        info.discount?.let {r.add(DisplayInfoModel(langString(R.string.max_discount)
            , langString(R.string.discount_value,it)))}
        info.discount_amount?.let {r.add(DisplayInfoModel(langString(R.string.discounted_price)
            , it.toRubly()))}
        return r
    }

}

enum class InfoShowType{
    COMMON_INFO,
    EXPLICATIONS
}
