package com.biganto.visual.roompark.presentation.screen.favorites

import android.view.View
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import butterknife.BindView
import com.biganto.visual.roompark.R
import com.biganto.visual.roompark.base.HeaderToolbarModel
import com.biganto.visual.roompark.base.RoomParkApplication
import com.biganto.visual.roompark.base.RoomParkMainActivity
import com.biganto.visual.roompark.conductor.BigantoBaseController
import com.biganto.visual.roompark.conductor.dialogs.change_handler.DialogChangeHandler
import com.biganto.visual.roompark.conductor.dialogs.tour_chooser.ChooseTourDialogController
import com.biganto.visual.roompark.domain.model.EstateModel
import com.biganto.visual.roompark.presentation.screen.estate.EstateScreenController
import com.biganto.visual.roompark.presentation.screen.favorites.util.FavoritesListAdapter
import com.biganto.visual.roompark.util.view_utils.grid.CeilsDecoration
import com.bluelinelabs.conductor.RouterTransaction
import com.bluelinelabs.conductor.changehandler.FadeChangeHandler
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import timber.log.Timber
import java.util.concurrent.TimeUnit
import javax.inject.Inject

/**
 * Created by Vladislav Bogdashkin on 30.09.2019.
 */

class FavoritesScreenController  :
    BigantoBaseController<FavoritesScreenViewState
        , FavoritesScreen
        , FavoritesScreenPresenter>()
    , FavoritesScreen {

    override fun injectDependencies() {
        getComponent()
    }

    @Inject
    override lateinit var injectedPresenter: FavoritesScreenPresenter

    @BindView(R.id.favorites_cards_recycler_view)
    lateinit var favoritesRecyclerView: RecyclerView

    override fun tourCardClicked(): Observable<EstateModel> =
        (favoritesRecyclerView.adapter as FavoritesListAdapter)
            .startTour
            .debounce(220L, TimeUnit.MILLISECONDS)
            .observeOn(AndroidSchedulers.mainThread())

    override fun onAttach(view: View) {
        super.onAttach(view)
        detachDisposable.addAll(
            (favoritesRecyclerView.adapter as FavoritesListAdapter)
                .onItemClicked
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe {
                    Timber.d("got card clicked $it")
                    router.pushController(RouterTransaction.with(EstateScreenController(it.id))
                        .popChangeHandler(FadeChangeHandler())
                        .pushChangeHandler(FadeChangeHandler())
                    )
                }
        )
    }

    private fun setToolbar(){
        favoritesRecyclerView.isNestedScrollingEnabled = false

        toolBar.setToolbar(
            HeaderToolbarModel(
                false,null
                ,resources?.getString(R.string.favorites),null)
        )
    }

    private fun bindRecycler() {
        favoritesRecyclerView.isNestedScrollingEnabled = true
        favoritesRecyclerView.layoutManager =
            LinearLayoutManager(activity, RecyclerView.VERTICAL, false)
        favoritesRecyclerView.adapter = FavoritesListAdapter()
        favoritesRecyclerView.itemAnimator = null

        if (favoritesRecyclerView.itemDecorationCount == 0)
            favoritesRecyclerView.addItemDecoration(
                CeilsDecoration(1
                , resources?.getDimensionPixelSize(R.dimen.ceil_grid_padding))
            )

    }

    override fun onViewBound(v: View) {
        bottomNavigationController.show()
        setToolbar()
        bindRecycler()
    }

    override fun render(viewState: FavoritesScreenViewState) {
        super.render(viewState)
        Timber.d("Render state $viewState")
        when(viewState){
            is FavoritesScreenViewState.Idle -> render(viewState)
            is FavoritesScreenViewState.FavoriteEstatesLoaded -> render(viewState)
            is FavoritesScreenViewState.SomeError -> render(viewState)
            is FavoritesScreenViewState.ToursLoaded -> render(viewState)
        }
    }

    private fun render(viewState: FavoritesScreenViewState.Idle){

    }

    private fun render(viewState: FavoritesScreenViewState.ToursLoaded) {
        router.pushController(RouterTransaction.with(
            ChooseTourDialogController(ArrayList(viewState.tours))
        )
            .popChangeHandler(DialogChangeHandler())
            .pushChangeHandler(DialogChangeHandler())
        )
    }

    private fun render(viewState: FavoritesScreenViewState.SomeError) =
        showError(viewState.exception)


    private fun render(viewState: FavoritesScreenViewState.FavoriteEstatesLoaded) {
        (favoritesRecyclerView.adapter as FavoritesListAdapter).addItems(viewState.items)
    }

    private fun getComponent() = DaggerFavoritesScreenComponent.factory()
        .create(RoomParkApplication.component,activity as RoomParkMainActivity)
        .inject(this)

    override fun getLayoutId(): Int = R.layout.favorites_screen



}