package com.biganto.visual.roompark.presentation.screen.feed_list

import com.biganto.visual.roompark.conductor.BigantoBasePresenter
import com.biganto.visual.roompark.domain.interactor.ArticlesInteractor
import com.biganto.visual.roompark.domain.model.ArticlePreviewModel
import com.biganto.visual.roompark.domain.model.SubscriptionModel
import com.biganto.visual.roompark.util.monades.ExceptionString
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import timber.log.Timber
import javax.inject.Inject
import javax.inject.Named

/**
 * Created by Vladislav Bogdashkin on 30.09.2019.
 */

private const val FEED_PAGE_SIZE = 10


class ArticlesScreenPresenter @Inject constructor(
    @Named(FEED_ALIAS) private val feedId:String,
    private val interactor: ArticlesInteractor
)
    : BigantoBasePresenter<ArticlesScreen, ArticlesScreenViewState>() {

    private val restoreModel = RestoreModel(mutableListOf(), null)

    override fun detachView() {
        super.detachView()
        restoreStateObservable.accept(ArticlesScreenViewState.RestoreView(restoreModel))
    }


    override fun defaultErrorViewStateHandler() =
        { e: ExceptionString -> ArticlesScreenViewState.SomeError(e) }

    override fun bindIntents() {
        Timber.d("feedId : $feedId")

        val onSubChecked = intent(ArticlesScreen::feedSubscription)
            .filter { restoreModel.sub != null }
            .flatMap { newState ->
                interactor.switchSubscription(restoreModel.sub!!, newState)
                    .andThen(
                        Observable.just<ArticlesScreenViewState>(
                            ArticlesScreenViewState.SubscriptionStatus(
                                newState
                            )
                        )
                    )
                    .onErrorReturn { ArticlesScreenViewState.SubscriptionError(!newState) }
            }

        val fetchSubscription = interactor.getSubscriptions(feedId)
            .doOnNext { Timber.w("got smthng: $it") }
            .doAfterNext { restoreModel.sub = it }
            .map<ArticlesScreenViewState> { ArticlesScreenViewState.SubscriptionStatus(it.state) }
            .startWith(Observable.just<ArticlesScreenViewState>(ArticlesScreenViewState.Idle()))

        val prefetchCards = interactor.fetchArticles(feedId)
            .map<ArticlesScreenViewState> { ArticlesScreenViewState.ArticlesLoaded(it.articles) }
            .startWith(Observable.just<ArticlesScreenViewState>(ArticlesScreenViewState.Idle()))

        val getNewArticlesPage = intent(ArticlesScreen::requsetsNewArticles)
            .flatMap {
                interactor.fetchArticlesPage(feedId, FEED_PAGE_SIZE, it)
                    .map<ArticlesScreenViewState> { articlePage ->
                        articlePage.articles
                            .asSequence()
                            .filter { !restoreModel.articles.contains(it) }
                            .toList()
                            .let { uniqueItems -> restoreModel.articles.addAll(uniqueItems) }
                        ArticlesScreenViewState.ArticlesScrollPage(articlePage.articles.toList())
                    }
                    .onErrorReturn(::parseError)
            }

        val state = Observable.mergeDelayError(
            arrayListOf(
                restoreStateObservable,
                prefetchCards,
                getNewArticlesPage,
                fetchSubscription,
                onSubChecked
            )
        )
            .doOnError { Timber.e(it) }
            .onErrorReturn(::parseError)
            .subscribeOn(Schedulers.io())
            .observeOn(AndroidSchedulers.mainThread())

        subscribeViewState(state.cast(ArticlesScreenViewState::class.java), ArticlesScreen::render)
    }
}



data class RestoreModel(
    var articles:MutableList<ArticlePreviewModel>,
    var sub: SubscriptionModel?
)