package com.biganto.visual.roompark.presentation.screen.settings

import android.annotation.SuppressLint
import android.view.View
import android.widget.ImageView
import androidx.core.widget.NestedScrollView
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import butterknife.BindView
import com.biganto.visual.roompark.R
import com.biganto.visual.roompark.base.HeaderToolbarModel
import com.biganto.visual.roompark.base.RoomParkApplication
import com.biganto.visual.roompark.base.RoomParkMainActivity
import com.biganto.visual.roompark.conductor.BigantoBaseController
import com.biganto.visual.roompark.domain.model.SubscriptionModel
import com.biganto.visual.roompark.presentation.screen.settings.util.CahcedListAdapter
import com.biganto.visual.roompark.presentation.screen.settings.util.PushListAdapter
import com.biganto.visual.roompark.presentation.screen.splash.SplashScreenController
import com.biganto.visual.roompark.util.extensions.bytesToSize
import com.biganto.visual.roompark.util.extensions.setGone
import com.biganto.visual.roompark.util.monades.ExceptionString
import com.bluelinelabs.conductor.RouterTransaction
import com.bluelinelabs.conductor.changehandler.FadeChangeHandler
import com.google.android.material.textview.MaterialTextView
import com.jakewharton.rxbinding3.view.clicks
import com.jakewharton.rxrelay2.BehaviorRelay
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import timber.log.Timber
import java.util.concurrent.TimeUnit
import javax.inject.Inject

/**
 * Created by Vladislav Bogdashkin on 30.09.2019.
 */

class SettingsScreenController :
    BigantoBaseController<SettingsScreenViewState
        , SettingsScreen
        , SettingsScreenPresenter>()
    , SettingsScreen {

    override fun signOut(): Observable<Int> =
        signOutButton.clicks()
            .map { Timber.d("Clicked sign out button"); 1 }
            .observeOn(AndroidSchedulers.mainThread())

    override fun clearCache(): Observable<Int> =
        clearCacheButton.clicks()
            .debounce ( 500, TimeUnit.MICROSECONDS )
            .map { Timber.d("Clicked clear cache button"); 1 }
            .observeOn(AndroidSchedulers.mainThread())

    private val refreshEmitter = BehaviorRelay.create<Int>()

    override fun refreshCacheInfo(): Observable<Int> = refreshEmitter


    override fun onSubscription(): Observable<SubscriptionModel> =
        (pushRecycler.adapter as PushListAdapter)
            .onItemClicked
            .map { it.subModel }
            .debounce ( 600, TimeUnit.MILLISECONDS )
            .observeOn(AndroidSchedulers.mainThread())

    override fun injectDependencies() {
        getComponent()
    }

    @Inject
    override lateinit var injectedPresenter: SettingsScreenPresenter


    @BindView(R.id.nestedScrollContainer)
    lateinit var scrollContainer:NestedScrollView

    @BindView(R.id.pushRecyclerView)
    lateinit var pushRecycler:RecyclerView

    @BindView(R.id.cachedRecyclerView)
    lateinit var cachedRecycler:RecyclerView

    @BindView(R.id.downloadToursText)
    lateinit var toursDownloaderTitle:MaterialTextView

    @BindView(R.id.downloadFlatCardsText)
    lateinit var flatDownloaderTitle:MaterialTextView

    @BindView(R.id.downloadToursIcon)
    lateinit var toursDownloaderButton:ImageView

    @BindView(R.id.downloadFlatCardsIcon)
    lateinit var flatDownloaderButton:ImageView

    @BindView(R.id.signOutButton)
    lateinit var signOutButton:MaterialTextView

    @BindView(R.id.clear_cache_button)
    lateinit var clearCacheButton:MaterialTextView

    @BindView(R.id.progress_lock_background)
    lateinit var progressShame:View

    private fun setToolbar(){
        toolBar.appBar.liftOnScrollTargetViewId = R.id.nestedScrollContainer
        pushRecycler.isNestedScrollingEnabled = false
        cachedRecycler.isNestedScrollingEnabled = false
        
        toolBar.setToolbar(
            HeaderToolbarModel(
                false,null
                ,resources?.getString(R.string.settings),null)
        )
    }

    private fun bindRecycler(){
        pushRecycler.isNestedScrollingEnabled = true
        pushRecycler.layoutManager =
            LinearLayoutManager(activity, RecyclerView.VERTICAL, false)
        pushRecycler.adapter = PushListAdapter()
        pushRecycler.itemAnimator = null

        cachedRecycler.isNestedScrollingEnabled = true
        cachedRecycler.layoutManager =
            LinearLayoutManager(activity, RecyclerView.VERTICAL, false)
        cachedRecycler.adapter = CahcedListAdapter()
        cachedRecycler.itemAnimator = null
    }

    override fun onViewBound(v: View) {
        bottomNavigationController.show()
        setToolbar()
        bindRecycler()
    }

    override fun render(viewState: SettingsScreenViewState) {
        super.render(viewState)
        Timber.d("Render state $viewState")
        when(viewState){
            is SettingsScreenViewState.Idle -> render(viewState)
            is SettingsScreenViewState.LoadSettingsList -> render(viewState)
            is SettingsScreenViewState.SomeError -> render(viewState)
            is SettingsScreenViewState.SignOut -> render(viewState)
            is SettingsScreenViewState.OnCacheDeleting -> render(viewState)
            is SettingsScreenViewState.LoadCachInfo -> render(viewState)
            is SettingsScreenViewState.LoadSubscriptions -> render(viewState)
            is SettingsScreenViewState.SubscriptionStatus -> render(viewState)
            is SettingsScreenViewState.SubscriptionError -> render(viewState)
        }
    }

    private fun render(viewState: SettingsScreenViewState.SomeError) =
        showError(viewState.exception)

    private fun render(viewState: SettingsScreenViewState.Idle){
            progressShame.setGone(true)
            clearCacheButton.setGone(false)
    }

    private fun render(viewState: SettingsScreenViewState.LoadSubscriptions){
        (pushRecycler.adapter as PushListAdapter).setItems(viewState.list)
    }

    private fun render(viewState: SettingsScreenViewState.LoadCachInfo){
        (cachedRecycler.adapter as CahcedListAdapter).setItems(viewState.list)
    }

    private fun render(viewState: SettingsScreenViewState.SubscriptionStatus){
        (pushRecycler.adapter as PushListAdapter).setSubState(viewState.subId,viewState.subState)
    }

    private fun render(viewState: SettingsScreenViewState.SubscriptionError){
        (pushRecycler.adapter as PushListAdapter).setSubState(viewState.subId,viewState.subState)
        if (viewState.subState)
            showError(ExceptionString(R.string.unsubscribe_error_message,null))
        else showError(ExceptionString(R.string.subscribe_error_message,null))
    }


    private fun render(viewState: SettingsScreenViewState.OnCacheDeleting){
        val isProgressed = viewState.progress>=1f
        progressShame.setGone(isProgressed)
        clearCacheButton.setGone(!isProgressed)
        refreshEmitter.accept(1)
    }

    private fun render(viewState: SettingsScreenViewState.SignOut){
        router.setRoot(RouterTransaction.with(SplashScreenController())
            .pushChangeHandler(FadeChangeHandler())
            .popChangeHandler(FadeChangeHandler())
        )
    }


    @SuppressLint("SetTextI18n")
    private fun render(viewState: SettingsScreenViewState.LoadSettingsList){

        toursDownloaderTitle.text = viewState.settings.offlineStoreData[0].title +
        "(${viewState.settings.offlineStoreData[0].amountBytes.bytesToSize()})"

        flatDownloaderTitle.text = viewState.settings.offlineStoreData[1].title +
                "(${viewState.settings.offlineStoreData[1].amountBytes.bytesToSize()})"
    }

    private fun getComponent() = DaggerSettingsScreenComponent.factory()
        .create(RoomParkApplication.component,activity as RoomParkMainActivity)
        .inject(this)

    override fun getLayoutId(): Int = R.layout.settings_screen

}