package com.biganto.visual.roompark.presentation.screen.settings

import android.content.Context
import com.biganto.visual.roompark.conductor.BigantoBasePresenter
import com.biganto.visual.roompark.domain.interactor.SettingsInteractor
import com.biganto.visual.roompark.domain.model.CachedDataModel
import com.biganto.visual.roompark.domain.model.SubscriptionModel
import com.biganto.visual.roompark.domain.model.TitledSubscriptionModel
import com.biganto.visual.roompark.util.monades.ExceptionString
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import timber.log.Timber
import java.util.concurrent.TimeUnit
import javax.inject.Inject

/**
 * Created by Vladislav Bogdashkin on 30.09.2019.
 */


class SettingsScreenPresenter @Inject constructor(
    private val interactor: SettingsInteractor,
    private val activity: Context
)
    : BigantoBasePresenter<SettingsScreen, SettingsScreenViewState>() {


    private val restoreModel = RestoreModel(mutableListOf(), arrayListOf(), arrayListOf())

    override fun detachView() {
        super.detachView()
        restoreStateObservable.accept(SettingsScreenViewState.RestoreView(restoreModel))
    }

    override fun defaultErrorViewStateHandler() =
        { e: ExceptionString -> SettingsScreenViewState.SomeError(e) }

    override fun bindIntents() {


        val onSubChecked = intent(SettingsScreen::onSubscription)
            .flatMap { sub ->
                interactor.switchSubscription(sub, !sub.state)
                    .andThen(
                        Observable.just<SettingsScreenViewState>(
                            SettingsScreenViewState.SubscriptionStatus(
                                sub.id, !sub.state
                            )
                        )
                            .doOnNext {
                                val ind = restoreModel
                                    .subs
                                    .indexOfFirst { it.subModel.id == sub.id }
                                restoreModel.subs[ind] =
                                    TitledSubscriptionModel(
                                        title = restoreModel.subs[ind].title,
                                        subModel = SubscriptionModel(
                                            topic = sub.topic,
                                            id = restoreModel.subs[ind].subModel.id,
                                            state = !sub.state
                                        )
                                    )
                            }

                    )
                    .doOnError { Timber.e(it) }
                    .onErrorReturn { SettingsScreenViewState.SubscriptionError(sub.id, sub.state) }
            }

        val fetchSettings = interactor.fetchSettings()
            .doOnNext { restoreModel.offlineStoreData = it.offlineStoreData.toMutableList() }
            .map { SettingsScreenViewState.LoadSettingsList(it) }

        val fetchCache = interactor.getCacheInfo()
            .doOnNext { cached -> cached.sortBy { it.id } }
            .doOnNext { restoreModel.cacheInfo = it }
            .map { SettingsScreenViewState.LoadCachInfo(it) }

        val fetchSubscriptions = interactor.getSubscriptions()
            .doOnNext { restoreModel.subs = it.toMutableList() }
            .map { SettingsScreenViewState.LoadSubscriptions(it) }

        val onSignOut = intent(SettingsScreen::signOut)
            .flatMap {
                interactor.signOut()
                    .andThen(Observable.just<SettingsScreenViewState>(SettingsScreenViewState.SignOut()))
                    .onErrorReturn(::parseError)
            }


        val refreshInfo = intent(SettingsScreen::refreshCacheInfo)
            .flatMap { fetchCache }

        val onClearCache = intent(SettingsScreen::clearCache)
            .flatMap {
                interactor.deleteCacheFiles()
                    .map<SettingsScreenViewState> {
                        Timber.d(" got progress: ${it.first} / ${it.second.toFloat()}")
                        SettingsScreenViewState.OnCacheDeleting(
                            it.first / it.second.toFloat()
                        )
                    }
                    .delay(500,TimeUnit.MILLISECONDS)
                    .startWith(SettingsScreenViewState.OnCacheDeleting(0f))
                    .doOnError { Timber.e(it) }
            }

        val state = Observable.mergeDelayError(
            arrayListOf(
                restoreStateObservable,
                fetchSettings,
                onSignOut,
                onClearCache,
                refreshInfo,
                fetchSubscriptions,
                fetchCache,
                onSubChecked
            )
        )
            .doOnError { Timber.e(it) }
            .subscribeOn(Schedulers.io())
            .onErrorReturn(::parseError)
            .observeOn(AndroidSchedulers.mainThread())

        subscribeViewState(state.cast(SettingsScreenViewState::class.java), SettingsScreen::render)
    }

}



data class RestoreModel(
    var subs: MutableList<TitledSubscriptionModel>,
    var cacheInfo:MutableList<CachedDataModel>,
    var offlineStoreData:MutableList<CachedDataModel>
)